#include "cluster_reader.h"
#include <assert.h>

int read_clusters(FILE *fp, int64_t n_clusters, Cluster *buf, int *n_left) {
    int iframe = 0;
    int nph = *n_left;

    size_t nph_read = 0;
    size_t nn = *n_left;
    size_t nr = 0;

    // read photons left from previous frame
    if (nph) {
        if (nph > n_clusters) {
            // if we have more photons left in the frame then photons to read we
            // read directly
            nn = n_clusters;
        } else {
            nn = nph;
        }
        nr += fread((void *)(buf + nph_read), sizeof(Cluster), nn, fp);
        nph_read += nn;
        *n_left = nph - nn;
    }
    if (nph_read < n_clusters) {
        // keep on reading frames and photons until reaching n_clusters
        while (fread(&iframe, sizeof(iframe), 1, fp)) {
            if (fread(&nph, sizeof(nph), 1, fp)) {
                if (nph > n_clusters - nph_read)
                    nn = n_clusters - nph_read;
                else
                    nn = nph;

                nr += fread((void *)(buf + nph_read), sizeof(Cluster), nn, fp);
                nph_read += nn;
                *n_left = nph - nn;
            }
            if (nph_read >= n_clusters)
                break;
        }
    }
    assert(nph_read <= n_clusters); // sanity check in debug mode
    return nph_read;
}

int analyze_clusters(int64_t n_clusters, Cluster *cin, ClusterAnalysis *cout) {

    int32_t tot2[4], t2max;
    char c;
    int32_t val, tot3;
    for (int ic = 0; ic < n_clusters; ic++) {
        tot3 = 0;
        for (int i = 0; i < 4; i++)
            tot2[i] = 0;
        // t2max=0;
        for (int ix = 0; ix < 3; ix++) {
            for (int iy = 0; iy < 3; iy++) {
                val = (cin + ic)->data[iy * 3 + ix];
                tot3 += val;
                if (ix <= 1 && iy <= 1)
                    tot2[0] += val;
                if (ix >= 1 && iy <= 1)
                    tot2[1] += val;
                if (ix <= 1 && iy >= 1)
                    tot2[2] += val;
                if (ix >= 1 && iy >= 1)
                    tot2[3] += val;
            }
        }
        t2max = tot2[0];
        c = cBottomLeft;
        for (int i = 1; i < 4; i++) {
            if (tot2[i] > t2max) {
                t2max = tot2[i];
                c = i;
            }
        }
        (cout + ic)->c = c;
        (cout + ic)->tot2 = t2max;
        (cout + ic)->tot3 = tot3;
        // printf("%d %d %d %d %d %d\n",ic,(cin+ic)->x, (cin+ic)->y,
        // (cout+ic)->c, (cout+ic)->tot2, (cout+ic)->tot3);
    }
    return n_clusters;
}
